import os
from pymacaron_unit import testcase


def load_port_host_token():
    """Find out which host:port to run acceptance tests against,
    using the environment variables PYM_SERVER_HOST, PYM_SERVER_PORT
    """

    server_host, server_port, token = (None, None, None)

    if 'PYM_SERVER_HOST' in os.environ:
        server_host = os.environ['PYM_SERVER_HOST']
    if 'PYM_SERVER_PORT' in os.environ:
        server_port = os.environ['PYM_SERVER_PORT']

    token = os.environ.get('PYM_JWT_TOKEN', None)

    if server_host:
        if server_host.startswith('http://'):
            server_host = server_host[7:]
        if server_host.startswith('https://'):
            server_host = server_host[8:]
        if server_host.endswith('/'):
            server_host = server_host[:-1]

    if not server_host or not server_port:
        raise Exception("Please set both of PYM_SERVER_HOST and PYM_SERVER_PORT envvironment variables")

    return (server_host, server_port, token)


class PyMacaronTestCase(testcase.PyMacaronTestCase):

    token = None

    def setUp(self):
        super().setUp()
        self.maxDiff = None
        self.host, self.port, self.token = load_port_host_token()
        proto = 'https' if self.port in (443, '443') else 'http'
        self.base_url = '%s://%s:%s' % (proto, self.host, self.port)

    def assertIsVersion(self, j):
        self.assertTrue(type(j['version']) is str)
        self.assertTrue(type(j['name']) is str)
        self.assertTrue(type(j['container']) is str)
        self.assertTrue(type(j['pym_env']) is str)

    def assertHasPing(self):
        return self.assertGetReturnOk('ping')

    def assertHasVersion(self, verify_ssl=True):
        j = self.assertGetReturnJson('version', 200, verify_ssl=verify_ssl)
        self.assertIsVersion(j)
        return j

    def assertHasAuthVersion(self, verify_ssl=True):
        self.assertGetReturnError('auth/version', 401, 'AUTHORIZATION_HEADER_MISSING', verify_ssl=verify_ssl)

        tests = [
            # header,  status code, json code
            ("", 401, 'AUTHORIZATION_HEADER_MISSING'),
            ("Bearer1234567890", 401, 'TOKEN_INVALID'),
            ("bearer foo bar", 401, 'TOKEN_INVALID'),
            ("Bearer 1234567890", 401, 'TOKEN_INVALID'),
        ]

        for t in tests:
            token, status, error = t
            self.assertGetReturnError('auth/version', status, error, token, verify_ssl=verify_ssl)

        j = self.assertGetReturnJson('auth/version', 200, "Bearer %s" % self.token, verify_ssl=verify_ssl)
        self.assertIsVersion(j)
        return j



import logging
import pprint
import os
from time import sleep
from pymacaron_core.swagger.apipool import ApiPool
from pymacaron.utils import get_container_version
from pymacaron.crash import report_error
from pymacaron.exceptions import PyMacaronException


log = logging.getLogger(__name__)


class MyFatalCustomError(PyMacaronException):
    code = 'FATAL_CUSTOM_ERROR'
    status = 543

class MyNonFatalCustomError(PyMacaronException):
    code = 'NON_FATAL_CUSTOM_ERROR'
    status = 401


ping_hooks = []

def add_ping_hook(hook):
    global ping_hooks
    assert callable(hook), "Ping hook %s should be a function" % str(hook)
    ping_hooks.append(hook)

def do_ping():
    log.debug("Replying ping:ok")
    v = ApiPool.ping.model.Ok()
    for h in ping_hooks:
        log.info("Calling ping hook %s" % str(h))
        h()
    return v

def do_version():
    """Return version details of the running server api"""
    v = ApiPool.ping.model.Version(
        name=ApiPool().current_server_name,
        version=ApiPool().current_server_api.get_version(),
        container=get_container_version(),
        pym_env=os.environ.get('PYM_ENV', ''),
    )
    log.info("/version: " + pprint.pformat(v))
    return v

def do_crash_internal_exception():
    raise Exception("Raising an internal exception")

def do_crash_pymacaron_exception():
    raise MyNonFatalCustomError("Raising a non-fatal custom error")

def do_crash_report_error():
    report_error("called crash/reporterror to test error reporting")
    return ApiPool.crash.model.Ok()

def do_crash_slow_call():
    sleep(6)
    return ApiPool.crash.model.Ok()

def do_crash_return_fatal_error_response():
    return MyFatalCustomError("endpoint returns an Error response").http_reply()

def do_crash_return_non_fatal_error_response():
    return MyNonFatalCustomError("endpoint returns a non-fatal Error response").http_reply()

def do_crash_return_error_model():
    return ApiPool.crash.model.Error(
        status=543,
        error='ANOTHER_CUSTOM_ERROR',
        error_description='Testing error model',
    )

def do_crash_return_error_instance():
    return MyFatalCustomError("endpoint returns an Error instance")



import os
import sys
import yaml
import pprint
import logging
from urllib.parse import urlparse


log = logging.getLogger(__name__)


class PyMacaronConfig(object):

    def __init__(self, path=None, env=None):

        # Some defaults, required by pymacaron.auth
        self.jwt_issuer = None
        self.jwt_audience = None
        self.jwt_secret = None
        self.jwt_token_timeout = 86400
        self.jwt_token_renew_after = 10800
        self.default_user_id = 'PYM_DEFAULT_USER_ID'

        # Default time-limit for the slow-call report
        self.report_call_exceeding_ms = 1000

        # Find a pym-config file
        pym_env = os.environ.get('PYM_ENV', None)
        if env:
            pym_env = env
        log.debug("Target environment is: [%s]" % pym_env)

        config_path = get_config_path('pym-config.%s.yaml' % pym_env, path=path)
        if not config_path:
            log.debug("Did not find pym-config.%s.yaml - Defaulting to searching for pym-config.yaml" % (pym_env))
            config_path = get_config_path('pym-config.yaml', path=path)

        if not config_path:
            raise Exception("Failed to find pym-config!")

        self.config_path = config_path

        log.info("Loading config file at %s" % config_path)
        all_keys = []
        config_dict = {}
        with open(config_path, 'r') as stream:

            # Support versions of PyYAML with and without Loader
            import pkg_resources
            v = pkg_resources.get_distribution("PyYAML").version
            if v > '3.15':
                config_dict = yaml.load(stream, Loader=yaml.FullLoader)
            else:
                config_dict = yaml.load(stream)

            for k, v in config_dict.items():
                setattr(self, k, v)
                all_keys.append(k)

        # Validate config
        if hasattr(self, 'live_url'):
            o = urlparse(self.live_url)
            host = o.netloc
            if ':' in host:
                host = host.split(':')[0]
            self.live_host = host

        if not hasattr(self, 'live_host'):
            raise Exception("'pym-config.yaml' lacks the 'live_host' or 'live_url' key")

        # Magic here :-)
        # For all keys whose value is in the list of enironment secrets, replace
        # that value with the value of the corresponding environment variable.
        if hasattr(self, 'env_secrets'):
            log.info("Substituting secret environment variable names for their values in config")
            for k in all_keys:
                if getattr(self, k) in self.env_secrets:
                    setattr(self, k, os.environ.get(getattr(self, k), k))
                    config_dict[k] = str(getattr(self, k))[0:8] + '****'

        # Print config file to log, but obfuscate secrets
        log.debug("Loaded configuration:\n%s" % pprint.pformat(config_dict, indent=4))


def get_config_path(name='pym-config.yaml', path=None):
    """Search for a file named 'name' in all the places where 'pym-config.*.yaml'
    is normally looked for.  Return a path if found, or None if not found
    """

    paths = [
        os.path.join(os.path.dirname(sys.argv[0]), name),
        '/pym/%s' % name,
        os.path.join(os.path.dirname(sys.argv[0]), 'test/%s' % name),
        os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', name),
        os.path.join(os.getcwd(), name),
    ]

    if path:
        if not path.endswith(name):
            path = os.path.join(path, name)
        paths.append(path)

    for p in paths:
        p = os.path.abspath(p)
        log.info("Looking for file %s at %s" % (name, p))
        if os.path.isfile(p):
            return p

    return None


config = None


def get_config(path=None, env=None):
    """Find the pym-config yaml file for the current environment, as identified by
    the PYM_ENV variable. Look at all standard locations. Optionally take an
    extra location/path to look at, and/or an environment that overrides
    PYM_ENV
    """

    global config
    if not config:
        config = PyMacaronConfig(path=path, env=env)
    return config
